"""Not sure what this is going to do yet.
    Probably managed unique GameEngine instances (one per game).
"""

from typing import Any

class CareersGameManager(object):

    def __init__(self):
        pass

    def __call__(self) -> Any:
        return self