'''
Created on Aug 06, 2022

@author: don_bacon
'''

VERSION_TUPLE = (0, 0, 6)
__version__ = ".".join(map(str, VERSION_TUPLE))
__title__ = 'server'
__description__ = 'Python Careers game'
__author__ = 'Donald Bacon'
__author_email__ = 'dwbzen@gmail.com'
__license__ = 'MIT License'
__copyright__ = 'Copyright 2022 Donald Bacon'